/**-----------------------------------------------------------------------------------------
* Copyright © 2025 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
import { DateFormat, FilterEditor, FilterOperator, NumberFormat } from './model/filter-expression';
import { FilterValueEditorTemplateDirective } from './templates/value-editor.template';
import * as i0 from "@angular/core";
/**
 * Represents the [Kendo UI Filter Field component for Angular]({% slug api_filter_filterfieldcomponent %}).
 * Used to declare filter expressions for the Filter component.
 *
 * @example
 * ```html
 * <kendo-filter>
 *   <kendo-filter-field field="country" editor="string" [operators]="['neq', 'eq', 'contains']"></kendo-filter-field>
 *   <kendo-filter-field field="budget" editor="number"></kendo-filter-field>
 *   <kendo-filter-field field="discontinued" title="Discontinued" editor="boolean"></kendo-filter-field>
 *   <kendo-filter-field field="ordered on" title="Ordered on" editor="date"></kendo-filter-field>
 * <kendo-filter>
 * ```
 */
export declare class FilterFieldComponent {
    /**
     * Specifies the `field` that will be used by the user-defined filter.
     */
    field: string;
    /**
     * Specifies the `title` text that will be displayed by the user-defined filter.
     * If the `title` isn't set, the value passed to `field` is used.
     */
    set title(_title: string);
    get title(): string;
    private _title;
    /**
     * Specifies the user-defined filter `editor` type that will be used.
     * The available options are 'string', 'number', 'boolean', and 'date'.
     */
    editor: FilterEditor;
    /**
     * Specifies the operators that will be available in the order of providing them.
     * If no operators are provided, default operators are used for each filter type.
     *
     * The default string operators are:
     * * `eq`&mdash; Is equal to
     * * `neq`&mdash; Is not equal to
     * * `isnull`&mdash; Is null
     * * `isnotnull`&mdash; Is not null
     * * `contains`&mdash; Contains
     * * `doesnotcontain`&mdash; Does not contain
     * * `startswith`&mdash; Starts with
     * * `endswith`&mdash; Ends with
     * * `isempty`&mdash; Is empty
     * * `isnotempty`&mdash; Is not empty
     *
     * The default number and date operators are:
     * * `eq`&mdash; Is equals to
     * * `neq`&mdash; Is not equal to
     * * `isnull`&mdash; Is null
     * * `isnotnull`&mdash; Is not null
     * * `gt`&mdash; Greater than
     * * `gte`&mdash; Greater than or equal to
     * * `lt`&mdash; Less than
     * * `lte`&mdash; Less than or equal to
     *
     * The boolean operator is always set to `eq`
     */
    operators: FilterOperator[];
    /**
     * Specifies the user-defined filter `editor` format that will be used. ([see example]({% slug filter_editor_formats %}))
     */
    editorFormat?: string | NumberFormat | DateFormat;
    /**
     * @hidden
     */
    editorTemplate: FilterValueEditorTemplateDirective;
    static ɵfac: i0.ɵɵFactoryDeclaration<FilterFieldComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FilterFieldComponent, "kendo-filter-field", never, { "field": { "alias": "field"; "required": false; }; "title": { "alias": "title"; "required": false; }; "editor": { "alias": "editor"; "required": false; }; "operators": { "alias": "operators"; "required": false; }; "editorFormat": { "alias": "editorFormat"; "required": false; }; }, {}, ["editorTemplate"], never, true, never>;
}
